"""
VendAsta Package Manager
"""
import os
import sys
import base64
import json
import logging
import getpass

from model import Package, Packages, Configuration
from github import Github, GithubException

CONFIG_FILE = 'vpmrc.json'
CONFIG_USERNAME = 'githubUsername'
CONFIG_PASSWORD = 'githubPassword'
PACKAGE_FILE = 'packages.json'
CACHE_ROOT = '.vpmcache'
INSTALL_PATH = 'installPath'
PACKAGES = 'packages'


def load_configuration():
    """
    Loads the VPM configuration.
    """
    try:
        configuration = Configuration.from_file()
    except (IOError, ValueError):
        username = raw_input("Github username: ")
        password = getpass.getpass("Github password: ")
        kwargs = {
            Configuration.GITHUB_USERNAME: username,
            Configuration.GITHUB_PASSWORD: password,
        }
        configuration = Configuration(**kwargs)

    return configuration

def load_package_list():
    """
    Loads the package configuration.
    """
    try:
        packages = Packages.from_file()
    except (IOError, ValueError) as exception:
        logging.critical("A package.json file was not found \
                or is ill formatted. %s", exception)
        sys.exit(1)

    return packages

# TODO: Tests
def get_sha_for_tag(repository, tag):
    """
    Returns a commit PyGithub object for the specified repository and tag.
    """
    branches = repository.get_branches()
    matched_branches = [match for match in branches if match.name == tag]
    if matched_branches:
        return matched_branches[0].commit.sha

    tags = repository.get_tags()
    matched_tags = [match for match in tags if match.name == tag]
    if not matched_tags:
        raise ValueError('No Tag or Branch exists with that name')
    return matched_tags[0].commit.sha

# TODO: Tests
def clear_cache():
    """
    Delete the contents of CACHE_ROOT.
    """
    if not os.path.exists(CACHE_ROOT):
        os.makedirs(CACHE_ROOT)

    os.popen('rm -rf ' + CACHE_ROOT + '/*')

# TODO: Tests
def download_package(repository, sha, server_path, download_path):
    """
    Download all contents at server_path with commit tag sha in 
    the repository.
    """
    contents = repository.get_dir_contents(server_path, ref=sha)

    for content in contents:
        print "Processing %s" % content.path
        if content.type == 'dir':
            dir_download_path = '%s/%s' % (download_path, content.name)
            os.makedirs(dir_download_path)
            download_package(repository, sha, content.path, dir_download_path)
        else:
            try:
                path = content.path
                file_content = repository.get_contents(path, ref=sha)
                file_data = base64.b64decode(file_content.content)
                file_out = open(download_path + '/' + content.name, "w")
                file_out.write(file_data)
                file_out.close()
            except (GithubException, IOError) as exc:
                logging.error('Error processing %s: %s', content.path, exc)


def vpm(package_name, tag=None):
    """
    VendAsta Package Manager.
    """
    # Load configuration
    print "Loading configuration"
    configuration = load_configuration()

    username = configuration.username
    password = configuration.password

    print "Loading package list"
    packages = load_package_list()

    print "Installing %s" % package_name

    print "Connecting to GitHub"
    github = Github(username, password)
    organization = github.get_user().get_orgs()[0]

    for package in packages:
        if package.name == package_name:
            repository = organization.get_repo(package.repository)

            sha = get_sha_for_tag(repository, tag)

            clear_cache()

            # Download package
            print "Downloading package contents"
            if not os.path.exists(package.download_path):
                os.makedirs(package.download_path)

            download_package(repository, sha, package.directory_path, 
                    package.download_path)

            # Install package
            print "Removing existing package"
            os.popen('rm -rf ' + package.install_path)

            if not os.path.exists(package.install_path):
                os.makedirs(package.install_path)

            print "Installing %s version %s" % (package.name, tag)
            os.popen('mv ' + package.download_path + '/*' + ' ' + package.install_path)

    print "%s with tag %s successfully installed." % (package_name, tag)

def run():
    """
    Main entry point.
    """
    args = sys.argv[1:]
    if not len(args) == 2:
        logging.error("Usage: vpm package version")
        return
    vpm(args[0], args[1])


if __name__ == '__main__':
    run()
